#ifndef DISPLAY_H
#define DISPLAY_H

// Stores the pattern to show on the display.
extern unsigned char display_data[4];

// Initialises the display pins and resets the segment scanning fields.
void display_init(void);

// Switches the display off.
void display_off(void);

// Switches the display on.
void display_on(void);

// Switches the display off when it has finished scanning the entire display. This allows for flicker-free drawing when needed.
void display_off_synchronised(void);

// Clear the display.
void display_clear(void);

// Outputs a single segment. This function should be called in a loop.
void display_output_segment(void);

// Outputs a complete display frame.
void display_output_frame(void);

// Converts a digit (in the range 0x0 to 0xF) to its segment pattern for a single space on the display.
unsigned char display_get_segments_for_digit(unsigned char digit);

// Prints a 16-bit value as hex onto the display.
void display_print_hex_word(unsigned int value);

// Prints a decimal (0000~9999) onto the display. If the MSB of the value is set, display padding zeroes.
void display_print_decimal(unsigned int value);

// Set all four digits on the display to the supplied byte values.
#define display_set_display(a,b,c,d) display_data[0] = ((unsigned char)a); display_data[1] = ((unsigned char)b); display_data[2] = ((unsigned char)c); display_data[3] = ((unsigned char)d);

// Prints a floating point value onto the display.
void display_print_float(float value);

#endif