#include <htc.h>
#include "../hardware/display.h"
#include "../hardware/buttons.h"
#include "../hardware/resistors.h"
#include "../modes/probe.h"

void probe(void) {

	// Clear the display.
	display_clear();

	// Set ADC clock to FOSC/32.
	ADCS1 = 1;
	
	// Left-align results.
	ADFM = 0;

	// Switch on the ADC.
	ADON = 1;

	display_off();
	di();

	// Use an external clock source (feeds the pulse stretcher).
	T0CS = 1;
	TMR0 = 255;

	unsigned char pulse_stretcher_display_timer = 0;

	// Run the voltmeter until both buttons are pressed.
	while (BUTTON_1 || BUTTON_2) {

		// Pull the input to the middle (forced floating) with one pull-up and one pull-down resistor.
		TRISC &= 0b00111111;
		PORTC = 0b01000000;

		display_output_frame();

		// Disconnect the two 10K resistors.
		TRISC |= 0b11000000;

		// Delay for a short while to take a reading.
		__delay_us(100);

		// Take the reading
		GODONE = 1;
		while (GODONE);
		unsigned char reading = ADRESH;

		if (reading < PROBE_THRESHOLD_LOW) {
			// Low.
			display_data[0] = 0b00111000;
			T0SE = 0;
		} else if (reading > PROBE_THRESHOLD_HIGH) {
			// High.
			display_data[0] = 0b01110110;
			T0SE = 1;
		} else {
			// Floating/High-Z.
			display_data[0] = 0b01000000;
		}
		
		// Handle the pulse stretcher.
		if (TMR0IF) {
			TMR0IF = 0;
			TMR0 = 255;
			pulse_stretcher_display_timer = 10;
		}
		// Has a pulse been detected recently?
		if (pulse_stretcher_display_timer > 0) {
			--pulse_stretcher_display_timer;
			display_data[1] = 0b01110011; // 'P'
		} else {
			display_data[1] = 0b00000000; // ' '
		}

		// Refresh the display for a short while.
		for (unsigned char display_delay = 0; display_delay < 50; ++display_delay) {
			display_output_frame();
		}
	}

	// Switch off the ADC.
	ADON = 0;

	// Interrupt-driven display back on.
	ei();
	display_on();

}