#include <htc.h>
#include <stdio.h>
#include "../hardware/display.h"
#include "../hardware/buttons.h"
#include "../hardware/resistors.h"
#include "../utility/delay.h"

enum resistance {
	RESISTANCE_5K,
	RESISTANCE_10K,
	RESISTANCE_100K,	
};

float probe_resistance;

void set_probe_resistance(enum resistance r) {
	resistors_disconnect_all();
	if (r == RESISTANCE_100K) {
		probe_resistance = 100e3f;
		TRIS_R100K = 0;
		R100K = 1;
	} else {
		probe_resistance = 10e3f;
		R10K = 1;
		TRIS_R10K = 0;
		if (r == RESISTANCE_5K) {
			probe_resistance = 5e3f;
			R10KA = 1;
			TRIS_R10KA = 0;
		}
	}
}

void resistance(void) {
	
	// Set ADC clock to FOSC/32.
	ADCS1 = 1;
	
	// Right-align results.
	ADFM = 1;

	// Switch on the ADC.
	ADON = 1;

	// Pull the probe high with a resistor.
	set_probe_resistance(RESISTANCE_10K);

	// Run the resistance meter until both buttons are pressed.
	while (BUTTON_1 || BUTTON_2) {

		display_off();
		delay_ms(2);
	
		// Take a reading.
		GODONE = 1;
		while (GODONE);

		unsigned int reading = (ADRESL | (ADRESH << 8));

		display_on();

		// Convert the reading into a resistance.

		// Vout = Rb / (Rt + Rb) * Vin
		// Rb = Rt / ((Vin/Vout) - 1)
		// Vin = 1023
		// Vout = reading
	
		if (reading == 1023) {
			display_set_display(0b00000000, 0b00000110, 0b00110111, 0b01110001); // Inf
		} else {
			float resistance = 0;

			if (reading != 0) resistance = probe_resistance / ((1023.0f / (float)reading) - 1.0f);
			
			// Adjust the probe resistance to cope with different ranges.
			if (resistance < 2e3f) {
				set_probe_resistance(RESISTANCE_5K);
			} else if (resistance < 100e3f) {
				set_probe_resistance(RESISTANCE_10K);
			} else {
				set_probe_resistance(RESISTANCE_100K);
			}

			// Display the result.
			display_print_float(resistance);

		}

		// Delay between readings.
		delay_ms(200);
	}

	// Switch off the ADC.
	ADON = 0;
}