#include <htc.h>

#include "../main/isr.h"
#include "../main/settings.h"
#include "../hardware/resistors.h"
#include "../hardware/buttons.h"
#include "../hardware/display.h"
#include "../utility/delay.h"

#define BUTTON_INCREASE_FREQUENCY BUTTON_1
#define BUTTON_DECREASE_FREQUENCY BUTTON_2

void update_frequency(void) {
	square_wave_frequency = settings.square_wave_frequency * 2;
	display_print_decimal(settings.square_wave_frequency);
	display_output_frame();
}

void increase_frequency(void) {
	if (settings.square_wave_frequency < 9999) {
		++settings.square_wave_frequency;
		update_frequency();
	}
}

void decrease_frequency(void) {
	if (settings.square_wave_frequency > 1) {
		--settings.square_wave_frequency;
		update_frequency();
	}
}

void square_wave(void) {
	
	// Output a square wave using TRIS_R20.
	TRIS_R20 = 0;
	
	// Use the special interrupt mode to generate square waves.
	update_frequency();
	square_wave_reference_frequency	= _XTAL_FREQ / 4 / 256;
	square_wave_accumulator = 0;
	tmr0_interrupt_mode = TMR0_INTERRUPT_MODE_SQUARE_WAVE;

	while (BUTTON_DECREASE_FREQUENCY || BUTTON_INCREASE_FREQUENCY) {

		// Allow the user to alter the frequency.
		unsigned int key_repeat_timer = 2000;
		unsigned char settings_changed = 0;
		unsigned int previous_frequency = settings.square_wave_frequency;

		// Decrease frequency?
		while (!BUTTON_DECREASE_FREQUENCY && BUTTON_INCREASE_FREQUENCY) {
			settings_changed = 1;
			decrease_frequency();
			while (key_repeat_timer-- && !BUTTON_DECREASE_FREQUENCY && BUTTON_INCREASE_FREQUENCY) {
				display_output_frame();
			}
			key_repeat_timer = (previous_frequency - settings.square_wave_frequency) > 100 ? 5 : 100;
		}
		// Increase frequency?
		while (!BUTTON_INCREASE_FREQUENCY && BUTTON_DECREASE_FREQUENCY) {
			settings_changed = 1;
			increase_frequency();
			while (key_repeat_timer-- && !BUTTON_INCREASE_FREQUENCY && BUTTON_DECREASE_FREQUENCY) {
				display_output_frame();
			}
			key_repeat_timer = (settings.square_wave_frequency - previous_frequency) > 100 ? 5 : 100;
		}

		// Save the settings if need be.
		if (settings_changed) settings_save();

		// Refresh the display.
		display_output_frame();
	}



	// Restore default interrupt mode.
	tmr0_interrupt_mode = TMR0_INTERRUPT_MODE_DISPLAY_OUTPUT_SEGMENT;
}