#ifndef ONE_WIRE_H
#define ONE_WIRE_H

// ROM command values.
#define ONE_WIRE_COMMAND_SEARCH_ROM   (0xF0)
#define ONE_WIRE_COMMAND_READ_ROM     (0x33)
#define ONE_WIRE_COMMAND_MATCH_ROM    (0x55)
#define ONE_WIRE_COMMAND_SKIP_ROM     (0xCC)
#define ONE_WIRE_COMMAND_ALARM_SEARCH (0xEC)

// Initialise the 1-Wire bus.
void one_wire_init(void);

// Write a bit to the 1-Wire bus.
void one_wire_write_bit(unsigned char value);

// Write a byte to the 1-Wire bus.
void one_wire_write_byte(unsigned char value);

// Write a byte to the 1-Wire bus, and leave the bus in a powered state (strong pull-ups).
void one_wire_write_byte_powered(unsigned char value);

// Read a bit from the 1-Wire bus.
unsigned char one_wire_read_bit(void);

// Read a byte from the 1-Wire bus.
unsigned char one_wire_read_byte(void);

// Reset the 1-Wire bus slave devices. Returns 0 if a device asserts its presence, non-zero otherwise.
unsigned char one_wire_reset(void);

// Calculate the CRC for a block of data.
unsigned char one_wire_calculate_crc(unsigned char* data, unsigned char length);

#endif